@@ -150,7 +150,7 @@ extern "C" SEXP RAdbcMoveDatabase(SEXP database_xptr) {
 }
 
 extern "C" SEXP RAdbcDatabaseValid(SEXP database_xptr) {
-  AdbcDatabase* database = adbc_from_xptr<AdbcDatabase>(database_xptr);
+  AdbcDatabase* database = adbc_from_xptr<AdbcDatabase>(database_xptr, true);
   return Rf_ScalarLogical(database != nullptr && database->private_data != nullptr);
 }
 
@@ -219,7 +219,7 @@ extern "C" SEXP RAdbcMoveConnection(SEXP connection_xptr) {
 }
 
 extern "C" SEXP RAdbcConnectionValid(SEXP connection_xptr) {
-  AdbcConnection* connection = adbc_from_xptr<AdbcConnection>(connection_xptr);
+  AdbcConnection* connection = adbc_from_xptr<AdbcConnection>(connection_xptr, true);
   return Rf_ScalarLogical(connection != nullptr && connection->private_data != nullptr);
 }
 
@@ -274,10 +274,11 @@ extern "C" SEXP RAdbcConnectionGetObjects(SEXP connection_xptr, SEXP depth_sexp,
                                           SEXP error_xptr) {
   auto connection = adbc_from_xptr<AdbcConnection>(connection_xptr);
   int depth = adbc_as_int(depth_sexp);
-  const char* catalog = adbc_as_const_char(catalog_sexp);
-  const char* db_schema = adbc_as_const_char(db_schema_sexp);
-  const char* table_name = adbc_as_const_char(table_name_sexp);
+  const char* catalog = adbc_as_const_char(catalog_sexp, true);
+  const char* db_schema = adbc_as_const_char(db_schema_sexp, true);
+  const char* table_name = adbc_as_const_char(table_name_sexp, true);
 
+  // Build the null-terminated const char** used to filter by table type
   int table_type_length = Rf_length(table_type_sexp);
   SEXP table_type_shelter =
       PROTECT(Rf_allocVector(RAWSXP, (table_type_length + 1) * sizeof(const char*)));
@@ -287,12 +288,22 @@ extern "C" SEXP RAdbcConnectionGetObjects(SEXP connection_xptr, SEXP depth_sexp,
   }
   table_type[table_type_length] = nullptr;
 
-  const char* column_name = adbc_as_const_char(column_name_sexp);
+  // Ensure that R_NilValue maps to null and not a null-termianted const char**
+  // of length 0.
+  const char** table_type_maybe_null;
+  if (table_type_sexp == R_NilValue) {
+    table_type_maybe_null = nullptr;
+  } else {
+    table_type_maybe_null = table_type;
+  }
+
+  const char* column_name = adbc_as_const_char(column_name_sexp, true);
   auto out_stream = adbc_from_xptr<ArrowArrayStream>(out_stream_xptr);
   auto error = adbc_from_xptr<AdbcError>(error_xptr);
 
-  int status = AdbcConnectionGetObjects(connection, depth, catalog, db_schema, table_name,
-                                        table_type, column_name, out_stream, error);
+  int status =
+      AdbcConnectionGetObjects(connection, depth, catalog, db_schema, table_name,
+                               table_type_maybe_null, column_name, out_stream, error);
   UNPROTECT(1);
   return adbc_wrap_status(status);
 }
@@ -396,7 +407,7 @@ extern "C" SEXP RAdbcMoveStatement(SEXP statement_xptr) {
 }
 
 extern "C" SEXP RAdbcStatementValid(SEXP statement_xptr) {
-  AdbcStatement* statement = adbc_from_xptr<AdbcStatement>(statement_xptr);
+  AdbcStatement* statement = adbc_from_xptr<AdbcStatement>(statement_xptr, true);
   return Rf_ScalarLogical(statement != nullptr && statement->private_data != nullptr);
 }
 
